<?php

/**
 * @var \Illuminate\Database\Eloquent\Collection $user
 */

use kernel\modules\user\models\User;
use Itguild\EloquentTable\ViewEloquentTable;
use Itguild\EloquentTable\ViewJsonTableEloquentModel;
use kernel\IGTabel\btn\DangerBtn;
use kernel\IGTabel\btn\PrimaryBtn;
use kernel\IGTabel\btn\SuccessBtn;
use kernel\widgets\IconBtn\IconBtnDeleteWidget;
use kernel\widgets\IconBtn\IconBtnEditWidget;
use kernel\widgets\IconBtn\IconBtnListWidget;

$table = new ViewEloquentTable(new ViewJsonTableEloquentModel($user, [
    'params' => ["class" => "table table-bordered", "border" => "2"],
    'baseUrl' => "/admin/user",
]));
$table->beforePrint(function () use ($user) {
    $btn = IconBtnEditWidget::create(['url' => '/admin/user/profile/edit'])->run();
    $btn .= IconBtnDeleteWidget::create(['url' => '/admin/user/delete/' . $user->id])->run();
    return $btn;
});

$entityRelation = new \kernel\EntityRelation();
$additionals = $entityRelation->getEntityAdditionalProperty("user", $user);

foreach ($additionals as $key => $additional) {
    $table->addRow($key, function () use ($additional) {
        return $additional;
    }, ['after' => 'email']);
}

$table->rows([
    'user_photo' => function ($data) {
        return $data ? "<img src='$data' width='300px'>" : "";
    },
    'created_at' => function ($data) {
        if (!$data){
            return null;
        }

        return (new DateTimeImmutable($data))->format("d-m-Y");
    },
    'updated_at' => function ($data) {
        if (!$data){
            return null;
        }

        return (new DateTimeImmutable($data))->format("d-m-Y");
    }
]);
$table->create();
$table->render();